import pytest
import logging
from pyskyq.skyq import SkyQ
from pyskyq.constants import REMOTE_PORT


@pytest.mark.parametrize("host,port,logger", [
    ("hostname", 5900, None),
    ("hostname", 2342, logging.getLogger()),
])
def test_SkyQ_init(host, port, logger):
    skybox = SkyQ(host, port, logger)
    assert skybox.host == host
    assert skybox.remote_port == port
    assert isinstance(skybox.logger, logging.Logger)




"""This module houses the main SKY Q class"""

import logging
from typing import Optional

from .constants import REMOTE_PORT
from .skyremote import SkyRemote

class SkyQ:
    """Main Sky Q class definition."""

    def __init__(self,
                 host: str,
                 remote_port: int = REMOTE_PORT,
                 logger: Optional[logging.Logger] = None,
                 ) -> None:
        
        """Initialise SkyQ object. """
        self.host = host
        self.remote_port = remote_port
        self.logger = logging.getLogger(__name__) if not logger else logger

        self.logger.debug(f"Initialised SkyQ object with host={host}, remote_port={remote_port}")
        self.remote = SkyRemote(self.host,self.remote_port, self.logger)



import pytest
import socket
import sys

from pyskyq.cli import run
from pyskyq import rcmd

def test_cli(mocker):

    m = mocker.patch('socket.socket')
    m.return_value.recv.side_effect = [  # set up the data to be returned on each successive call of socket.recv()
        b'SKY 000.001\n',
        b'\x01\x01',
        b'\x00\x00\x00\x00',
        b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
    ]

    mocker.patch.object(sys, 'argv', ['pyskyq', 'play'])
    
    run()

    m.assert_called_with(socket.AF_INET, socket.SOCK_STREAM)

    assert m.return_value.recv.call_count == 4
    m.return_value.send.assert_any_call(b'SKY 000.001\n')
    m.return_value.send.assert_any_call(b'\x01')
    m.return_value.send.assert_any_call(b'\x00')
    m.return_value.send.assert_any_call(bytearray(b'\x04\x00\x00\x00\x00\x00\xe4\x00'))

    # TODO test with --verbose



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Cli wrapper for the skyq API
"""

import argparse
import sys
import logging

from pyskyq import __version__, SkyQ, REMOTE_COMMANDS

__author__ = "Bradley Wood"
__copyright__ = "Bradley Wood"
__license__ = "mit"

_logger = logging.getLogger(__name__)


def parse_args(args):
    """Parse command line parameters

    Args:
      args ([str]): command line parameters as list of strings

    Returns:
      :obj:`argparse.Namespace`: command line parameters namespace
    """
    parser = argparse.ArgumentParser(
        description="Sends a command to a SkyQ box")
    parser.add_argument(
        '--version',
        action='version',
        version='pyskyq {ver}'.format(ver=__version__))
    parser.add_argument(
        dest="cmd",
        help="command to send",
        type=str,
        metavar="CMD")
    parser.add_argument(
        '-v',
        '--verbose',
        dest="loglevel",
        help="set loglevel to INFO",
        action='store_const',
        const=logging.INFO)
    parser.add_argument(
        '-vv',
        '--very-verbose',
        dest="loglevel",
        help="set loglevel to DEBUG",
        action='store_const',
        const=logging.DEBUG)
    return parser.parse_args(args)


def setup_logging(loglevel):
    """Setup basic logging

    Args:
      loglevel (int): minimum loglevel for emitting messages
    """
    logformat = "[%(asctime)s] %(levelname)s:%(name)s:%(message)s"
    logging.basicConfig(level=loglevel, stream=sys.stdout,
                        format=logformat, datefmt="%Y-%m-%d %H:%M:%S")


def main(args):
    """Main entry point allowing external calls

    Args:
      args ([str]): command line parameter list
    """
    args = parse_args(args)
    setup_logging(args.loglevel)
    _logger.debug("Starting SkyQ...")
    skyq = SkyQ('skyq')
    skyq.remote.send_command(REMOTE_COMMANDS[args.cmd])
    _logger.info("Script ends here")


def run():
    """Entry point for console_scripts
    """
    main(sys.argv[1:])


if __name__ == "__main__":
    run()



"""This module implements the SkyRemote class"""

import logging
import socket
import math
from typing import Optional, Callable

from .constants import REMOTE_PORT


class SkyRemote:
    """Main SkyRemote implementation"""

    def __init__(self,
                 host: str,
                 port: int = REMOTE_PORT,
                 logger: Optional[logging.Logger] = None,
                 ) -> None:
        """Initial Sky Remote Object."""
        self.host = host
        self.port = port
        self.logger = logging.getLogger(__name__) if not logger else logger
        self.logger.debug(f"Initialised SkyRemote object with host={host}, port={port}")

    def send_command(self, code: int) -> None:
        """Send a command to the Sky Q box using the Remote interface."""
        self.logger.debug(f'Sending command code={code}')
        command_bytes = bytearray([4, 1, 0, 0, 0, 0, math.floor(224 + (code/16)), code % 16])
        self.logger.debug(f'command_bytes={command_bytes}')

        # TODO try/except socket.gaierror 
        client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        client.connect((self.host, self.port))
        
        # Wait until ready for data
        length = 12
        while True:
            data = client.recv(24)
            self.logger.debug(f'Received data={data}')
            if len(data) < 24: # box sends 24 x \x00 to signal end of hand-shake.
                client.send(data[0:length])
                self.logger.debug(f'Sent data={data[0:length]}')
                length = 1
            else:
                break
        
        client.send(command_bytes)
        self.logger.debug(f'Send command part 1 data={command_bytes}')
        command_bytes[1] = 0
        client.send(command_bytes)
        self.logger.debug(f'Send command part 2 data={command_bytes}')
        client.close()



"""Constants for button codes, ports, and so on"""
from collections import namedtuple

REMOTE_LEGACY_PORT: int = 5900
REMOTE_PORT: int = 49160

REMOTE_COMMANDS = {
	'power': 0,
	'select': 1,
	'backup': 2,
	'dismiss': 2,
	'channelup': 6,
	'channeldown': 7,
	'interactive': 8,
	'sidebar': 8,
	'help': 9,
	'services': 10,
	'search': 10,
	'tvguide': 11,
	'home': 11,
	'i': 14,
	'text': 15,
	'up': 16,
	'down': 17,
	'left': 18,
	'right': 19,
	'red': 32,
	'green': 33,
	'yellow': 34,
	'blue': 35,
	'zero': 48,
	'one': 49,
	'two': 50,
	'three': 51,
	'four': 52,
	'five': 53,
	'six': 54,
	'seven': 55,
	'eight': 56,
	'nine': 57,
	'play': 64,
	'pause': 65,
	'stop': 66,
	'record': 67,
	'fastforward': 69,
	'rewind': 71,
	'boxoffice': 240,
	'sky': 241,
}

RCmdTuple = namedtuple('RCmdTuple', sorted(REMOTE_COMMANDS))

rcmd = RCmdTuple(**REMOTE_COMMANDS)



import pytest
import socket

from pyskyq import SkyQ, SkyRemote, rcmd


def test_sky_remote_init():
    skyq = SkyQ('blah')
    assert isinstance(skyq.remote, SkyRemote)
    assert skyq.remote.host == 'blah'

def test_sky_remote_send_command(mocker):
    
    m = mocker.patch('socket.socket')
    m.return_value.recv.side_effect = [  # set up the data to be returned on each successive call of socket.recv()
        b'SKY 000.001\n',
        b'\x01\x01',
        b'\x00\x00\x00\x00',
        b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
    ]

    skyrem = SkyRemote('blah')

    skyrem.send_command(rcmd.play)

    m.assert_called_with(socket.AF_INET, socket.SOCK_STREAM)
    
    assert m.return_value.recv.call_count == 4
    m.return_value.send.assert_any_call(b'SKY 000.001\n')
    m.return_value.send.assert_any_call(b'\x01')
    m.return_value.send.assert_any_call(b'\x00')
    m.return_value.send.assert_any_call(bytearray(b'\x04\x00\x00\x00\x00\x00\xe4\x00'))
